from __future__ import print_function
import sys
import click
import json
import csv
import requests
import urllib
from hashlib import sha1
import hmac
import time

import android

def get_token(secret,userid=None):

    if userid is None:
        msg = tohash = "%08x" % time.time()
    else:
        msg = ""
        tohash = userid
    hashed = hmac.new(secret,tohash,sha1)
    hashed = hashed.digest().encode('hex')
    return msg + hashed

@click.group()
def entrypoint():
    pass

@entrypoint.command()
@click.option('--variant', default='android', help='resource collection method')
@click.option('--output', help='filename to write output to (stdout if omitted)')
@click.option('--rootdir', help='where to look for resource files ("." if omitted)',default=".")
def pack(variant,output,rootdir):
    "Prepare upload pack for JITT"
    pack = globals().get(variant).create_pack(rootdir)
    if output is None:
        output = sys.stdout
    else:
        output = file(output,'w')
    json.dump(pack,output)

@entrypoint.command()
@click.argument('apikey')
@click.argument('secret')
@click.option('--packfile', help='upload pack file name (will read from stdin if omitted)')
@click.option('--server', help='JITT server to upload data to', default='http://jitt.io')
def upload(apikey,secret,packfile,server):
    "upload pack to JITT Server"
    if packfile is None:
        packfile = sys.stdin
    else:
        packfile = file(packfile)
    token = get_token(secret.encode('utf8'))
    url = server.rstrip('/') + '/api/upload'
    try:
        resp = requests.post(url,params={'apikey':apikey,'token':token},data=packfile.read())
        if resp.status_code == 200:
            print(resp.text)
        else:
            print('Error %d: %s' % (resp.status_code,resp.reason),file=sys.stderr)
    except Exception,e:
        print(e,file=sys.stderr)

@entrypoint.command()
@click.argument('apikey')
@click.argument('secret')
@click.option('--packfile', help='download pack file name (will write to stdout if omitted)')
@click.option('--server', help='JITT server to download data from', default='http://jitt.io')
def download(apikey,secret,packfile,server):
    "download pack from JITT Server"
    if packfile is None:
        packfile = sys.stdin
    else:
        packfile = file(packfile)
    token = get_token(secret.encode('utf8'))
    url = server.rstrip('/') + '/api/download'
    try:
        resp = requests.get(url,params={'apikey':apikey,'token':token})
        if resp.status_code == 200:
            print(resp.text)
        else:
            print('Error %d: %s' % (resp.status_code,resp.reason),file=sys.stderr)
    except Exception,e:
        print(e,file=sys.stderr)

@entrypoint.command()
@click.option('--variant', default='android', help='resource collection method')
@click.option('--input', help='filename to read input from (stdin if omitted)')
@click.option('--rootdir', help='where to look for resource files to update ("." if omitted)',default=".")
def unpack(variant,input,rootdir):
    "Merge translated pack from JITT into existing resource files"
    if input is None:
        input = sys.stdin
    else:
        input = file(input)
    pack = json.load(input)
    globals().get(variant).process_pack(rootdir,pack)

@entrypoint.command()
@click.argument('apikey')
@click.argument('secret')
@click.argument('userid')
@click.argument('locale')
@click.option('--server', help='JITT server to upload data to', default='http://jitt.io')
def link(apikey,secret,userid,locale,server):
    "Get translation link for a user"
    token = get_token(secret.encode('utf8'),userid)
    server = server.rstrip('/')
    url = '%s/#/start/%s/%s/%s/%s' % (server,apikey,locale,token,userid)
    print(url)

@entrypoint.group()
def status():
    "Get status on resources and users"
    pass

def flatten(recs,path):

    def copy_dict(rec):
        out = {}
        for k,v in rec.items():
            if type(v) is dict:
                for k1,v1 in copy_dict(v).items():
                    out["%s.%s" % (k,k1)] = v1
            elif type(v) is list:
                pass
            else:
                if type(v) is unicode:
                    v = v.encode('utf8')
                out[k] = v
        return out

    for rec in recs:
        if len(path) > 0:
            first = path[0]
            rest = path[1:]
            divein = rec.get(first,[])
            if first in rec:
                del rec[first]
            for inner in flatten(divein,rest):
                out = copy_dict(rec)
                for k,v in inner.items():
                    out["%s.%s" % (first,k)] = v
                yield out
        else:
            out = copy_dict(rec)
            yield out

@status.command()
@click.argument('apikey')
@click.argument('secret')
@click.option('--server', help='JITT server to fetch data from', default='http://jitt.io')
@click.option('--format', help='Output format (csv/json), default is json', default='json')
@click.option('--output', help='Output filename, default is stdout')
def resources(apikey,secret,server,format,output=None):
    "Get app's resource status"
    token = get_token(secret.encode('utf8'))
    fileinfo = requests.get('%s/api/admin/files/%s' % (server,apikey), {'token':token}).json()
    files = fileinfo['files'].keys()
    locales = fileinfo['locales']
    res = []
    for fn in files:
        for locale in locales:
            resources = requests.get('%s/api/admin/resources/%s/%s/%s' % (server,apikey,locale,urllib.quote_plus(fn)), {'token':token}).json()
            for resource in resources:
                res.append({u'filename':fn,'resources':resources})
    if output is None:
        output = sys.stdout
    else:
        output = open(output,"w")
    if format == 'json':
        json.dump(res,output)
    elif format == "csv":
        records = flatten(res,['resources','suggestions'])
        writer=csv.DictWriter(output,[
                     u'filename',
                     u'resources.resource_id',
                     u'resources.text',
                     u'resources.context',
                     u'resources.priority',
                     u'resources.created_at',
                     u'resources.stats.last_suggestion_added',
                     u'resources.stats.skipped',
                     u'resources.stats.unclear',
                     u'resources.stats.weight',
                     u'resources.suggestions.created_at',
                     u'resources.suggestions.locale',
                     u'resources.suggestions.text',
                     u'resources.suggestions.score',
                     ],extrasaction='ignore')
        writer.writeheader()
        writer.writerows(records)

@status.command()
@click.argument('apikey')
@click.argument('secret')
@click.option('--server', help='JITT server to fetch data from', default='http://jitt.io')
@click.option('--format', help='Output format (csv/json), default is json', default='json')
@click.option('--output', help='Output filename, default is stdout')
def users(apikey,secret,server,format,output):
    "Get app's user status"
    token = get_token(secret.encode('utf8'))
    users = requests.get('%s/api/admin/users/%s' % (server,apikey), {'token':token}).json()
    if output is None:
        output = sys.stdout
    else:
        output = open(output,"w")
    if format == 'json':
        json.dump(users,output)
    elif format == 'csv':
        records = flatten(users,['localeStats'])
        writer=csv.DictWriter(output,[u'userid',
                     u'created_at',
                     u'last_activity',
                     u'stats.approved',
                     u'stats.received',
                     u'stats.served',
                     u'stats.translated',
                     u'localeStats.locale',
                     u'localeStats.approved',
                     u'localeStats.received',
                     u'localeStats.served',
                     u'localeStats.translated',
                     ],extrasaction='ignore')
        writer.writeheader()
        writer.writerows(records)



import os
import json
from traverse import traverse, PLURALS

def create_pack(root_dir="."):
    strings = {}
    
    def add_rec(el,subel,kind,designator,locale,name,canonic_name,text,filename,priority,context):

        if kind == 'quantity':
            allowed_quantities = ['zero','one','two','few','many','other']
            if locale is not None:
                allowed_quantities = PLURALS.get(locale,allowed_quantities)
                if designator not in allowed_quantities:
                    return
            for quantity in allowed_quantities:
                cn = '::'.join( canonic_name.split('::')[:-1] + [quantity] )
                if cn not in strings:
                    rec = {
                        'filename':filename,
                        'name':cn,
                        'priority':priority,
                        'context':context,
                        'original':u"%s (%s)" % (text,quantity),
                        'locales':{}
                    }
                    strings[cn] = rec

        if locale is None:
            rec = {
                'filename':filename,
                'name':canonic_name,
                'priority':priority,
                'context':context,
                'original':text,
                'locales':{}
            }
            if canonic_name not in strings or strings.get(canonic_name).get('original') is None:
                strings.setdefault(canonic_name,{}).update(rec)
        else:
            strings.setdefault(canonic_name,{}).setdefault('locales',{})[locale]=text
        return False

    traverse(root_dir,add_rec)

    strings = list(strings.values())
    strings = filter(lambda x:x.has_key('name'),strings)
    return strings



VERSION = "0.1.11"
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

config = {
    'description': 'JITT Command Line Tool',
    'author': 'Adam Kariv',
    'url': 'http://jitt.it',
    'download_url': 'https://github.com/EverythingMe/jitter/tarball/'+VERSION,
    'author_email': 'adam@everything.me',
    'version': VERSION,
    'install_requires': [
        'click',
        'nose',
        'lxml',
    ],
    'package_data':{
        'jitter':['android/*json']
    },
    'packages': ['jitter', 'jitter.android'],
    'scripts': [],
    'name': 'jitter',
    'entry_points':{
        'console_scripts': [
            'jitter = jitter.cli:entrypoint',
        ]
    },
}

setup(**config)



import os
import json
import itertools

from lxml import etree

from traverse import traverse, indent

BASE_XML = '<?xml version="1.0" encoding="utf-8"?><resources xmlns:tools="http://schemas.android.com/tools" xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2"></resources>'

def process_pack(root_dir,server_pack):
    server_pack = dict((x['name'],x) for x in server_pack)
    update_pack = {}

    def remove_unused(el,subel,kind,designator,locale,name,canonic_name,text,filename,priority,context):
        if locale is None:
            if canonic_name in server_pack:
                update_pack[canonic_name] = server_pack[canonic_name]
        return False

    def check_rec(el,subel,kind,designator,locale,name,canonic_name,text,filename,priority,context):
        modified = False
        if locale is not None:
            localized = update_pack.get(canonic_name,{}).get('locales',{})
            text = localized.get(locale)
            if text is not None:
                if subel.text != text:
                    subel.text = text
                    print canonic_name,"<--",locale,"<--",text
                    modified = True
                del localized[locale]
                if len(localized)==0:
                    del update_pack[canonic_name]

        return modified

    traverse(root_dir,remove_unused)
    traverse(root_dir,check_rec)

    to_write = sorted(update_pack.values(),key=lambda x:(x['filename'],x['name']))
    to_write = itertools.groupby(to_write,lambda x:x['filename'])
    for filename,resources in to_write:
        filename = filename.lstrip(os.path.sep)
        filename = os.path.join(root_dir,filename)
        for res in resources:
            for locale, text in res.get('locales').iteritems():
                localized_filename = filename.replace(os.path.sep+'values'+os.path.sep,os.path.sep+'values-%s' % locale+os.path.sep)
                # Create the file if needed
                if not os.path.isfile(localized_filename):
                    if not os.path.exists(os.path.dirname(localized_filename)):
                        os.makedirs(os.path.dirname(localized_filename))
                    dummy = open(localized_filename,'w')
                    dummy.write(BASE_XML)
                    dummy.close()
                # Now, open (or re-open) the localized file and parse it
                parser = etree.XMLParser(remove_blank_text=True)
                tree = etree.parse(localized_filename,parser)
                root = tree.getroot()
                assert(root.tag=='resources')
                # Find out if this is a regular resource, string-array or plurals
                name_parts = res['name'].split('::')
                parent = None
                new_el = None
                if len(name_parts) == 1:
                    new_el = etree.fromstring('<string name="%s"></string>' % name_parts[0])
                    new_el.text = text
                    print name_parts[0],"<-",text
                    parent=root
                elif len(name_parts) == 3:
                    if name_parts[1] == 'A':
                        # Array
                        new_el = etree.fromstring('<item></item>')
                        new_el.text = text
                        # Find parent
                        array_name=name_parts[0]
                        parent = root.xpath("string-array[@name='%s']" % array_name)
                        if len(parent)==0:
                            # Not found - create a new array element
                            parent = etree.fromstring('<string-array name="%s"></string-array>' % array_name)
                            parent.append(new_el)
                            new_el = parent
                            parent = root
                        else:
                            parent = parent[0]
                    elif name_parts[1] == 'P':
                        # plurals
                        new_el = etree.fromstring('<item quantity="%s"></item>' % name_parts[2])
                        new_el.text = text
                        # Find parent
                        plural_name=name_parts[0]
                        parent = root.xpath("plurals[@name='%s']" % plural_name)
                        if len(parent)==0:
                            # Not found - create a new plurals element
                            parent = etree.fromstring('<plurals name="%s"></plurals>' % plural_name)
                            parent.append(new_el)
                            new_el = parent
                            parent = root
                        else:
                            parent = parent[0]
                # make sure we have a known resource
                if parent is None or new_el is None:
                    raise RuntimeException("Bad string type: %s" % res['name'])
                # Make the necessary change
                parent.append(new_el)
                # Bail out
                indent(root)
                out = open(localized_filename, 'w')
                out.write(etree.tostring(root,encoding='UTF-8'))
                out.close()



import os
import json
from lxml import etree

LOCALES = json.load(file(os.path.join(os.path.dirname(__file__),'locales.json')))
PLURALS = json.load(file(os.path.join(os.path.dirname(__file__),'plurals.json')))

def indent(elem, level=0):
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i

def traverse(rootdir, visitor):

    for dirpath, dirnames, filenames in os.walk(rootdir):
        locale = None
        lastpart = os.path.split(dirpath)[1]
        path_splits = set(dirpath.split(os.path.sep))
        avoided = set(['build','bin'])
        if len(path_splits.intersection(avoided)) > 0:
            continue
        if lastpart.startswith('values'):
            parts = lastpart.split('-')
            if len(parts)==1:
                locale=None
            else:
                locale = parts[1]
                if locale not in LOCALES:
                    continue
            for fn in filter(lambda x:x.endswith('.xml'),filenames):
                modified = False
                fn = os.path.join(dirpath,fn)
                original_filename = fn.replace(rootdir,'')
                parser = etree.XMLParser(remove_blank_text=True)
                tree = etree.parse(fn,parser)
                root = tree.getroot()
                if root.tag!='resources':
                    continue
                globalcontext = None
                for el in root:
                    if el.tag == 'string':
                        name = el.get('name')
                        priority = el.get('priority',4)
                        context = el.get('context',globalcontext)
                        translatable = el.get('translatable') != 'false'
                        text = el.text
                        if translatable:
                            modified = visitor(el,el,None,None,locale,name,name,text,original_filename,priority,context) or modified
                    elif el.tag == 'string-array':
                        array_name = el.get('name')
                        array_priority = el.get('priority',4)
                        array_context = el.get('context',globalcontext)
                        array_translatable = el.get('translatable')
                        items = el.xpath('item')
                        for index,item in enumerate(el):
                            priority = item.get('priority',array_priority)
                            context = item.get('context',array_context)
                            translatable = item.get('translatable',array_translatable) != 'false'
                            text = item.text
                            if translatable:
                                canonic_name = "%s::A::%03d" % (array_name,index)
                                modified = visitor(el,item,'array',index,locale,array_name,canonic_name,text,original_filename,priority,context) or modified
                    elif el.tag == 'plurals':
                        plural_name = el.get('name')
                        plural_priority = el.get('priority',4)
                        plural_context = el.get('context',globalcontext)
                        plural_translatable = el.get('translatable')
                        for item in el:
                            quantity = item.get('quantity')
                            priority = item.get('priority',plural_priority)
                            context = item.get('context',plural_context)
                            translatable = item.get('translatable',plural_translatable) != 'false'
                            text = item.text
                            if translatable:
                                canonic_name = "%s::P::%s" % (plural_name,quantity)
                                modified = visitor(el,item,'quantity',quantity,locale,plural_name,canonic_name,text,original_filename,priority,context) or modified
                    elif el.tag == etree.Comment:
                        globalcontext = el.text
                    else:
                        pass

                if modified:
                    indent(root)
                    out = open(fn, 'w')
                    out.write(etree.tostring(root,encoding='UTF-8'))
                    out.close()
